import { HashMap, Exception, StdAsyncCallback } from "../common";
import { IEventEmitter, EventListenerConfig } from "./base";
/**
 * Signature for an asynchronous listener callback.
 */
export interface AsyncEventListener {
    (this: AsyncEventEmitter, ...args: any[]): void;
}
/**
 * Signature for listeners chaining callback.
 */
export interface AsyncEventChainCallback {
    (err?: Exception, breaked?: boolean): void;
}
export interface AsyncEventListenerConfig extends EventListenerConfig<AsyncEventListener> {
    /**
     * This must be a function.
     */
    "listener": AsyncEventListener;
}
/**
 * This class is a based-class for event emitter.
 * All classes based on this class could obtain the events handling engine, in
 * asynchronous way.
 */
export declare class AsyncEventEmitter extends IEventEmitter<AsyncEventListener> {
    protected _events: HashMap<AsyncEventListenerConfig[]>;
    constructor();
    /**
     * Register a listener to specific event.
     * The new listener will be prepend at the begin of listeners queue.
     */
    listen(name: string, fn: AsyncEventListener): AsyncEventEmitter;
    /**
     * Register a once-listener to specific event.
     * The new listener will be prepend at the begin of listeners queue.
     */
    listenOnce(name: string, fn: AsyncEventListener): AsyncEventEmitter;
    /**
     * Register a listener to specific event.
     */
    on(name: string, fn: AsyncEventListener): AsyncEventEmitter;
    /**
     * Register a once-listener to specific event.
     */
    once(name: string, fn: AsyncEventListener): AsyncEventEmitter;
    /**
     * Trigger/Emit a event.
     */
    emit(name: string, args?: any[], callback?: StdAsyncCallback): void;
}
