"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var base_1 = require("./base");
var async = require("async");
/**
 * This class is a based-class for event emitter.
 * All classes based on this class could obtain the events handling engine, in
 * asynchronous way.
 */
var AsyncEventEmitter = (function (_super) {
    __extends(AsyncEventEmitter, _super);
    function AsyncEventEmitter() {
        _super.call(this);
        this._events = {};
    }
    /**
     * Register a listener to specific event.
     * The new listener will be prepend at the begin of listeners queue.
     */
    AsyncEventEmitter.prototype.listen = function (name, fn) {
        if (this._events[name] === undefined) {
            this._events[name] = [];
        }
        this._events[name].unshift({
            "listener": fn
        });
        return this;
    };
    /**
     * Register a once-listener to specific event.
     * The new listener will be prepend at the begin of listeners queue.
     */
    AsyncEventEmitter.prototype.listenOnce = function (name, fn) {
        if (this._events[name] === undefined) {
            this._events[name] = [];
        }
        this._events[name].unshift({
            "once": true,
            "listener": fn
        });
        return this;
    };
    /**
     * Register a listener to specific event.
     */
    AsyncEventEmitter.prototype.on = function (name, fn) {
        if (this._events[name] === undefined) {
            this._events[name] = [];
        }
        this._events[name].push({
            "listener": fn
        });
        return this;
    };
    /**
     * Register a once-listener to specific event.
     */
    AsyncEventEmitter.prototype.once = function (name, fn) {
        if (this._events[name] === undefined) {
            this._events[name] = [];
        }
        this._events[name].push({
            "once": true,
            "listener": fn
        });
        return this;
    };
    /**
     * Trigger/Emit a event.
     */
    AsyncEventEmitter.prototype.emit = function (name, args, callback) {
        if (args === void 0) { args = []; }
        var listenerConfigs = this._events[name];
        if (listenerConfigs === undefined) {
            callback && setImmediate.apply(void 0, [callback].concat(args));
            return;
        }
        var i = 0;
        async.whilst(function () {
            return i < listenerConfigs.length;
        }, function (next) {
            var listener = listenerConfigs[i].listener;
            if (listenerConfigs[i].once) {
                listenerConfigs.splice(i--, 1);
            }
            args.push(function (err, breaked) {
                if (err) {
                    return next(err);
                }
                if (breaked) {
                    i = listenerConfigs.length;
                }
                args.pop();
                i++;
                next();
            });
            setImmediate.apply(void 0, [listener.bind(this)].concat(args));
        }.bind(this), function (err) {
            callback && callback(err);
        });
    };
    return AsyncEventEmitter;
}(base_1.IEventEmitter));
exports.AsyncEventEmitter = AsyncEventEmitter;
//# sourceMappingURL=async-events.js.map