import { HashMap } from "../common";
export interface EventListenerConfig<T> {
    /**
     * Specify whether this listener will be called only once.
     */
    "once"?: boolean;
    /**
     * This must be a function.
     */
    "listener": T;
}
export declare abstract class IEventEmitter<T> {
    protected _events: HashMap<EventListenerConfig<T>[]>;
    /**
     * Aka method `on`, this is an alias， for the compatibilities.
     */
    addEventListener: (name: string, fn: T) => IEventEmitter<T>;
    constructor();
    /**
     * Get the names list of events.
     */
    readonly events: string[];
    /**
     * Get the number of listeners to an event.
     */
    countListeners(name: string): number;
    /**
     * Unbind a listener to a specific event.
     * If no listener is specified, all listeners to that event will be removed.
     */
    unbind(name: string, fn?: T): number;
    /**
     * Register a listener to specific event.
     */
    abstract on(name: string, fn: T): IEventEmitter<T>;
    /**
     * Register a once-listener to specific event.
     */
    abstract once(name: string, fn: T): IEventEmitter<T>;
    /**
     * Register a listener to specific event.
     * The new listener will be prepend at the begin of listeners queue.
     */
    abstract listen(name: string, fn: T): IEventEmitter<T>;
    /**
     * Register a once-listener to specific event.
     * The new listener will be prepend at the begin of listeners queue.
     */
    abstract listenOnce(name: string, fn: T): IEventEmitter<T>;
}
