"use strict";
var IEventEmitter = (function () {
    function IEventEmitter() {
        this.addEventListener = this.on;
    }
    Object.defineProperty(IEventEmitter.prototype, "events", {
        /**
         * Get the names list of events.
         */
        get: function () {
            var rtn = [];
            for (var name_1 in this._events) {
                rtn.push(name_1);
            }
            return rtn;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Get the number of listeners to an event.
     */
    IEventEmitter.prototype.countListeners = function (name) {
        if (this._events[name] === undefined) {
            return 0;
        }
        return this._events[name].length;
    };
    /**
     * Unbind a listener to a specific event.
     * If no listener is specified, all listeners to that event will be removed.
     */
    IEventEmitter.prototype.unbind = function (name, fn) {
        var lst = this._events[name];
        if (lst === undefined) {
            return 0;
        }
        if (fn) {
            for (var x = void 0; x < lst.length; x++) {
                if (lst[x].listener === fn) {
                    lst.splice(x, 1);
                    return 1;
                }
            }
        }
        else {
            var deleted = void 0;
            deleted = this._events[name].length;
            delete this._events[name];
            return deleted;
        }
    };
    return IEventEmitter;
}());
exports.IEventEmitter = IEventEmitter;
//# sourceMappingURL=base.js.map