import { HashMap } from "../common";
import { IEventEmitter, EventListenerConfig } from "./base";
export interface SyncEventListener {
    (this: SyncEventEmitter, ...args: any[]): any;
}
export interface SyncEventListenerConfig extends EventListenerConfig<SyncEventListener> {
    /**
     * This must be a function.
     */
    "listener": SyncEventListener;
}
/**
 * This class is a based-class for event emitter.
 * All classes based on this class could obtain the events handling engine, in
 * synchronous way.
 */
export declare class SyncEventEmitter extends IEventEmitter<SyncEventListener> {
    protected _events: HashMap<SyncEventListenerConfig[]>;
    constructor();
    /**
     * Register a listener to specific event.
     * The new listener will be prepend at the begin of listeners queue.
     */
    listen(name: string, fn: SyncEventListener): SyncEventEmitter;
    /**
     * Register a once-listener to specific event.
     * The new listener will be prepend at the begin of listeners queue.
     */
    listenOnce(name: string, fn: SyncEventListener): SyncEventEmitter;
    /**
     * Register a listener to specific event.
     */
    on(name: string, fn: SyncEventListener): SyncEventEmitter;
    /**
     * Register a once-listener to specific event.
     */
    once(name: string, fn: SyncEventListener): SyncEventEmitter;
    /**
     * Trigger/Emit a event.
     */
    emit(name: string, args?: any[]): void;
}
