"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var base_1 = require("./base");
/**
 * This class is a based-class for event emitter.
 * All classes based on this class could obtain the events handling engine, in
 * synchronous way.
 */
var SyncEventEmitter = (function (_super) {
    __extends(SyncEventEmitter, _super);
    function SyncEventEmitter() {
        _super.call(this);
        this._events = {};
    }
    /**
     * Register a listener to specific event.
     * The new listener will be prepend at the begin of listeners queue.
     */
    SyncEventEmitter.prototype.listen = function (name, fn) {
        if (this._events[name] === undefined) {
            this._events[name] = [];
        }
        this._events[name].unshift({
            "listener": fn
        });
        return this;
    };
    /**
     * Register a once-listener to specific event.
     * The new listener will be prepend at the begin of listeners queue.
     */
    SyncEventEmitter.prototype.listenOnce = function (name, fn) {
        if (this._events[name] === undefined) {
            this._events[name] = [];
        }
        this._events[name].unshift({
            "once": true,
            "listener": fn
        });
        return this;
    };
    /**
     * Register a listener to specific event.
     */
    SyncEventEmitter.prototype.on = function (name, fn) {
        if (this._events[name] === undefined) {
            this._events[name] = [];
        }
        this._events[name].push({
            "listener": fn
        });
        return this;
    };
    /**
     * Register a once-listener to specific event.
     */
    SyncEventEmitter.prototype.once = function (name, fn) {
        if (this._events[name] === undefined) {
            this._events[name] = [];
        }
        this._events[name].push({
            "once": true,
            "listener": fn
        });
        return this;
    };
    /**
     * Trigger/Emit a event.
     */
    SyncEventEmitter.prototype.emit = function (name, args) {
        var listenerConfigs = this._events[name];
        if (listenerConfigs === undefined) {
            return;
        }
        for (var i = 0; i < listenerConfigs.length; i++) {
            var result = listenerConfigs[i].listener.apply(this, args);
            if (listenerConfigs[i].once) {
                listenerConfigs.splice(i--, 1);
            }
            if (result === false) {
                return;
            }
        }
    };
    return SyncEventEmitter;
}(base_1.IEventEmitter));
exports.SyncEventEmitter = SyncEventEmitter;
//# sourceMappingURL=sync-events.js.map