"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var events = require("../events");
var NodeURL = require("url");
var NodeHTTP = require("http");
var async = require("async");
var common_1 = require("../common");
var SmartRouteRule = require("./SmartRouteRule");
(function (ServerStatus) {
    ServerStatus[ServerStatus["IDLE"] = 0] = "IDLE";
    ServerStatus[ServerStatus["STARTING"] = 1] = "STARTING";
    ServerStatus[ServerStatus["WORKING"] = 2] = "WORKING";
    ServerStatus[ServerStatus["CLOSING"] = 3] = "CLOSING";
    ServerStatus[ServerStatus["CLOSED"] = 4] = "CLOSED";
    ServerStatus[ServerStatus["ERROR"] = 5] = "ERROR";
})(exports.ServerStatus || (exports.ServerStatus = {}));
var ServerStatus = exports.ServerStatus;
(function (HttpStatusCode) {
    HttpStatusCode[HttpStatusCode["CONTINUE"] = 100] = "CONTINUE";
    HttpStatusCode[HttpStatusCode["SWITCHING_PROTOCOLS"] = 101] = "SWITCHING_PROTOCOLS";
    HttpStatusCode[HttpStatusCode["PROCESSING"] = 102] = "PROCESSING";
    HttpStatusCode[HttpStatusCode["OK"] = 200] = "OK";
    HttpStatusCode[HttpStatusCode["CREATED"] = 201] = "CREATED";
    HttpStatusCode[HttpStatusCode["ACCEPTED"] = 202] = "ACCEPTED";
    HttpStatusCode[HttpStatusCode["NON_AUTHORITATIVE_INFORMATION"] = 203] = "NON_AUTHORITATIVE_INFORMATION";
    HttpStatusCode[HttpStatusCode["NO_CONTENT"] = 204] = "NO_CONTENT";
    HttpStatusCode[HttpStatusCode["RESET_CONTENT"] = 205] = "RESET_CONTENT";
    HttpStatusCode[HttpStatusCode["PARTIAL_CONTENT"] = 206] = "PARTIAL_CONTENT";
    HttpStatusCode[HttpStatusCode["MULTIPLE_CHOICES"] = 300] = "MULTIPLE_CHOICES";
    HttpStatusCode[HttpStatusCode["MOVED_PERMANENTLY"] = 301] = "MOVED_PERMANENTLY";
    HttpStatusCode[HttpStatusCode["MOVE_TEMPORARILY"] = 302] = "MOVE_TEMPORARILY";
    HttpStatusCode[HttpStatusCode["SEE_OTHER"] = 303] = "SEE_OTHER";
    HttpStatusCode[HttpStatusCode["NOT_MODIFIED"] = 304] = "NOT_MODIFIED";
    HttpStatusCode[HttpStatusCode["USE_PROXY"] = 305] = "USE_PROXY";
    HttpStatusCode[HttpStatusCode["SWITCH_PROXY"] = 306] = "SWITCH_PROXY";
    HttpStatusCode[HttpStatusCode["TEMPORARY_REDIRECT"] = 307] = "TEMPORARY_REDIRECT";
    HttpStatusCode[HttpStatusCode["BAD_REQUEST"] = 400] = "BAD_REQUEST";
    HttpStatusCode[HttpStatusCode["UNAUTHORIZED"] = 401] = "UNAUTHORIZED";
    HttpStatusCode[HttpStatusCode["FORBIDDEN"] = 403] = "FORBIDDEN";
    HttpStatusCode[HttpStatusCode["NOT_FOUND"] = 404] = "NOT_FOUND";
    HttpStatusCode[HttpStatusCode["METHOD_NOT_ALLOWED"] = 405] = "METHOD_NOT_ALLOWED";
    HttpStatusCode[HttpStatusCode["NOT_ACCEPTABLE"] = 406] = "NOT_ACCEPTABLE";
    HttpStatusCode[HttpStatusCode["PROXY_AUTHENTICATION_REQUIRED"] = 407] = "PROXY_AUTHENTICATION_REQUIRED";
    HttpStatusCode[HttpStatusCode["REQUEST_TIMEOUT"] = 408] = "REQUEST_TIMEOUT";
    HttpStatusCode[HttpStatusCode["CONFLICT"] = 409] = "CONFLICT";
    HttpStatusCode[HttpStatusCode["GONE"] = 410] = "GONE";
    HttpStatusCode[HttpStatusCode["LENGTH_REQUIRED"] = 411] = "LENGTH_REQUIRED";
    HttpStatusCode[HttpStatusCode["PRECONDITION_FAILED"] = 412] = "PRECONDITION_FAILED";
    HttpStatusCode[HttpStatusCode["REQUEST_ENTITY_TOO_LARGE"] = 413] = "REQUEST_ENTITY_TOO_LARGE";
    HttpStatusCode[HttpStatusCode["REQUEST_URI_TOO_LONG"] = 414] = "REQUEST_URI_TOO_LONG";
    HttpStatusCode[HttpStatusCode["UNSUPPORTED_MEDIA_TYPE"] = 415] = "UNSUPPORTED_MEDIA_TYPE";
    HttpStatusCode[HttpStatusCode["REQUESTED_RANGE_NOT_SATISFIABLE"] = 416] = "REQUESTED_RANGE_NOT_SATISFIABLE";
    HttpStatusCode[HttpStatusCode["EXPECTATION_FAILED"] = 417] = "EXPECTATION_FAILED";
    HttpStatusCode[HttpStatusCode["UNPROCESSABLE_ENTITY"] = 422] = "UNPROCESSABLE_ENTITY";
    HttpStatusCode[HttpStatusCode["LOCKED"] = 423] = "LOCKED";
    HttpStatusCode[HttpStatusCode["FAILED_DEPENDENCY"] = 424] = "FAILED_DEPENDENCY";
    HttpStatusCode[HttpStatusCode["UNORDERED_COLLECTION"] = 425] = "UNORDERED_COLLECTION";
    HttpStatusCode[HttpStatusCode["UPGRADE_REQUIRED"] = 426] = "UPGRADE_REQUIRED";
    HttpStatusCode[HttpStatusCode["RETRY_WITH"] = 449] = "RETRY_WITH";
    HttpStatusCode[HttpStatusCode["INTERNAL_SERVER_ERROR"] = 500] = "INTERNAL_SERVER_ERROR";
    HttpStatusCode[HttpStatusCode["NOT_IMPLEMENTED"] = 501] = "NOT_IMPLEMENTED";
    HttpStatusCode[HttpStatusCode["BAD_GATEWAY"] = 502] = "BAD_GATEWAY";
    HttpStatusCode[HttpStatusCode["SERVICE_UNAVAILABLE"] = 503] = "SERVICE_UNAVAILABLE";
    HttpStatusCode[HttpStatusCode["GATEWAY_TIMEOUT"] = 504] = "GATEWAY_TIMEOUT";
    HttpStatusCode[HttpStatusCode["HTTP_VERSION_NOT_SUPPORTED"] = 505] = "HTTP_VERSION_NOT_SUPPORTED";
    HttpStatusCode[HttpStatusCode["VARIANT_ALSO_NEGOTIATES"] = 506] = "VARIANT_ALSO_NEGOTIATES";
    HttpStatusCode[HttpStatusCode["INSUFFICIENT_STORAGE"] = 507] = "INSUFFICIENT_STORAGE";
    HttpStatusCode[HttpStatusCode["BANDWIDTH_LIMIT_EXCEEDED"] = 509] = "BANDWIDTH_LIMIT_EXCEEDED";
    HttpStatusCode[HttpStatusCode["NOT_EXTENDED"] = 510] = "NOT_EXTENDED";
    HttpStatusCode[HttpStatusCode["UNPARSEABLE_RESPONSE_HEADERS"] = 600] = "UNPARSEABLE_RESPONSE_HEADERS";
})(exports.HttpStatusCode || (exports.HttpStatusCode = {}));
var HttpStatusCode = exports.HttpStatusCode;
var DEFAULT_HOST = "127.0.0.1";
var DEFAULT_PORT = 80;
var DEFAULT_BACKLOG = 511;
/**
 * Method for HTTP request if SSL enabled.
 */
(function (HTTPStatusWithSSL) {
    HTTPStatusWithSSL[HTTPStatusWithSSL["ENABLED"] = 0] = "ENABLED";
    HTTPStatusWithSSL[HTTPStatusWithSSL["DISABLED"] = 1] = "DISABLED";
    HTTPStatusWithSSL[HTTPStatusWithSSL["REDIRECT"] = 2] = "REDIRECT";
})(exports.HTTPStatusWithSSL || (exports.HTTPStatusWithSSL = {}));
var HTTPStatusWithSSL = exports.HTTPStatusWithSSL;
;
/**
 * List of all supported methods.
 */
var AVAILABLE_METHODS = [
    "GET", "POST", "PATCH", "PUT",
    "DELETE", "HEAD", "OPTIONS", "ALL"
];
/**
 * Defines the router-table for each method supported.
 */
var RouterTable = (function () {
    function RouterTable() {
    }
    return RouterTable;
}());
/**
 * This class provides the properties for server.
 */
var ServerProperties = (function (_super) {
    __extends(ServerProperties, _super);
    function ServerProperties(options) {
        _super.call(this);
        this.inStatus = ServerStatus.IDLE;
        if (!options) {
            options = {};
        }
        this.inBacklog = options.backlog !== undefined ? options.backlog : DEFAULT_BACKLOG;
        this.inHost = options.host !== undefined ? options.host : DEFAULT_HOST;
        this.inPort = options.port !== undefined ? options.port : DEFAULT_PORT;
    }
    Object.defineProperty(ServerProperties.prototype, "host", {
        /**
         * The host of server.
         */
        get: function () {
            return this.inHost;
        },
        set: function (v) {
            if (this.inStatus === ServerStatus.IDLE) {
                this.inHost = v;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ServerProperties.prototype, "port", {
        /**
         * The port of server.
         */
        get: function () {
            return this.inPort;
        },
        set: function (v) {
            if (this.inStatus === ServerStatus.IDLE) {
                this.inPort = v;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ServerProperties.prototype, "backlog", {
        /**
         * The backlog of server.
         */
        get: function () {
            return this.inBacklog;
        },
        set: function (v) {
            if (this.inStatus === ServerStatus.IDLE) {
                this.inBacklog = v;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ServerProperties.prototype, "status", {
        /**
         * The status of server.
         */
        get: function () {
            return this.inStatus;
        },
        enumerable: true,
        configurable: true
    });
    return ServerProperties;
}(events.AsyncEventEmitter));
exports.ServerProperties = ServerProperties;
var Server = (function (_super) {
    __extends(Server, _super);
    function Server(options) {
        _super.call(this, options);
        // Initialize the router table.
        this.stringRouter = new RouterTable();
        this.advanceRouter = new RouterTable();
        for (var _i = 0, AVAILABLE_METHODS_1 = AVAILABLE_METHODS; _i < AVAILABLE_METHODS_1.length; _i++) {
            var key = AVAILABLE_METHODS_1[_i];
            this.stringRouter[key] = {};
            this.advanceRouter[key] = [];
        }
        this.middlewares = [];
    }
    /**
     * Register a handler for GET method.
     */
    Server.prototype.get = function (rule, handler) {
        if (typeof rule === "string") {
            this.registerStringRouter("GET", rule, handler);
        }
        else if (rule instanceof RegExp) {
            this.registerRegExpRouter("GET", rule, handler);
        }
        return this;
    };
    /**
     * Register a handler for POST method.
     */
    Server.prototype.post = function (rule, handler) {
        if (typeof rule === "string") {
            this.registerStringRouter("POST", rule, handler);
        }
        else if (rule instanceof RegExp) {
            this.registerRegExpRouter("POST", rule, handler);
        }
        return this;
    };
    /**
     * Register a handler for PUT method.
     */
    Server.prototype.put = function (rule, handler) {
        if (typeof rule === "string") {
            this.registerStringRouter("PUT", rule, handler);
        }
        else if (rule instanceof RegExp) {
            this.registerRegExpRouter("PUT", rule, handler);
        }
        return this;
    };
    /**
     * Register a handler for PATCH method.
     */
    Server.prototype.patch = function (rule, handler) {
        if (typeof rule === "string") {
            this.registerStringRouter("PATCH", rule, handler);
        }
        else if (rule instanceof RegExp) {
            this.registerRegExpRouter("PATCH", rule, handler);
        }
        return this;
    };
    /**
     * Register a handler for DELETE method.
     */
    Server.prototype.delete = function (rule, handler) {
        if (typeof rule === "string") {
            this.registerStringRouter("DELETE", rule, handler);
        }
        else if (rule instanceof RegExp) {
            this.registerRegExpRouter("DELETE", rule, handler);
        }
        return this;
    };
    /**
     * Register a handler for HEAD method.
     */
    Server.prototype.head = function (rule, handler) {
        if (typeof rule === "string") {
            this.registerStringRouter("HEAD", rule, handler);
        }
        else if (rule instanceof RegExp) {
            this.registerRegExpRouter("HEAD", rule, handler);
        }
        return this;
    };
    /**
     * Register a handler for OPTIONS method.
     */
    Server.prototype.options = function (rule, handler) {
        if (typeof rule === "string") {
            this.registerStringRouter("OPTIONS", rule, handler);
        }
        else if (rule instanceof RegExp) {
            this.registerRegExpRouter("OPTIONS", rule, handler);
        }
        return this;
    };
    /**
     * Register a handler for all methods if no others matched.
     */
    Server.prototype.all = function (rule, handler) {
        if (typeof rule === "string") {
            this.registerStringRouter("ALL", rule, handler);
        }
        else if (rule instanceof RegExp) {
            this.registerRegExpRouter("ALL", rule, handler);
        }
        return this;
    };
    /**
     * Register handler for Smart-Route rule.
     */
    Server.prototype.registerSmartRouter = function (method, exp, handler) {
        this.advanceRouter[method].push({
            "smartExp": exp,
            "handler": handler
        });
    };
    /**
     * Register handler for String rule.
     */
    Server.prototype.registerStringRouter = function (method, path, handler) {
        if (path.length > 1 && path[path.length - 1] === "/") {
            path = path.substr(0, path.length - 1);
        }
        if (path.indexOf("{") > -1) {
            this.registerSmartRouter(method, new SmartRouteRule(path), handler);
        }
        else {
            this.stringRouter[method][path] = handler;
        }
    };
    /**
     * Register handler for RegExp rule.
     */
    Server.prototype.registerRegExpRouter = function (method, exp, handler) {
        this.advanceRouter[method].push({
            "regExp": exp,
            "handler": handler
        });
    };
    // --------- Router End ---------
    // --------- Middleware Begin ---------
    /**
     * Setup a middleware for server.
     */
    Server.prototype.use = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        if (args.length === 0) {
            throw new common_1.Exception("BAD-MIDDLEWARE", "Invalid middleware registration.");
        }
        switch (args.length) {
            case 0:
                throw new common_1.Exception("BAD-MIDDLEWARE", "Invalid middleware registration.");
            case 1:
                if (args[0] instanceof Function) {
                    this.middlewares.push({
                        "handler": args[0]
                    });
                }
                else {
                    throw new common_1.Exception("BAD-MIDDLEWARE", "Invalid middleware registration.");
                }
                break;
            case 2:
            default:
                if ((args[0] instanceof RegExp || typeof (args[0]) === "string") && args[1] instanceof Function) {
                    if (typeof (args[0]) === "string") {
                        if (args[0].indexOf("{") > 0) {
                            args[0] = new SmartRouteRule(args[0]);
                        }
                    }
                    this.middlewares.push({
                        "expr": args[0],
                        "handler": args[1]
                    });
                }
                else {
                    throw new common_1.Exception("BAD-MIDDLEWARE", "Invalid middleware registration.");
                }
                break;
        }
        return this;
    };
    // --------- Middleware End ---------
    Server.create = function (options) {
        var server = function (req, resp) {
            server.gateway(req, resp);
        };
        Server.apply(server);
        Object.setPrototypeOf(server, Server.prototype);
        return server;
    };
    /**
     * Start the server.
     */
    Server.prototype.start = function (cb) {
        var _this = this;
        this.server = NodeHTTP.createServer(this.gateway.bind(this));
        this.server.once("error", function (err) {
            _this.emit("error", [
                new common_1.Exception("START-LISTEN", "Failed to listen to server")
            ], cb);
        });
        this.server.listen(this.port, this.host, this.backlog, cb);
    };
    Server.prototype.setupRequest = function (req) {
        var url = NodeURL.parse(req.url);
        req.rawPath = url.pathname;
        req.queryString = url.query;
        req.params = {};
        if (url.pathname.length === 1) {
            req.path = "/";
        }
        else {
            if (req.rawPath[req.rawPath.length - 1] === "/") {
                req.path = req.rawPath.substr(0, req.rawPath.length - 1);
            }
            else {
                req.path = req.rawPath;
            }
            if (req.path[0] !== "/") {
                req.path = "/" + req.path;
            }
        }
    };
    Server.prototype.gateway = function (req, resp) {
        this.setupRequest(req);
        if (!this.stringRouter[req.method]) {
            this.emit("/http/405", [req, resp, this], function (err) {
                resp.finished || resp.end();
            });
            return;
        }
        req.handler = this.stringRouter[req.method][req.path];
        if (!req.handler) {
            for (var _i = 0, _a = this.advanceRouter[req.method]; _i < _a.length; _i++) {
                var x = _a[_i];
                if (x.regExp) {
                    if (req.path.match(x.regExp)) {
                        req.handler = x.handler;
                        break;
                    }
                }
                else {
                    if (x.smartExp.match(req.path, req.params)) {
                        req.handler = x.handler;
                        break;
                    }
                }
            }
        }
        if (this.middlewares) {
            async.forEachOfSeries(this.middlewares, function (item, key, next) {
                if (item.expr) {
                    if (typeof (item.expr) === "string") {
                        if (item.expr !== req.path) {
                            next();
                            return;
                        }
                    }
                    else if (item.expr instanceof SmartRouteRule) {
                        if (!item.expr.match(req.path, req.params)) {
                            next();
                            return;
                        }
                    }
                    else if (item.expr instanceof RegExp) {
                        if (!req.path.match(item.expr)) {
                            next();
                            return;
                        }
                    }
                }
                item.handler(req, resp, function (err, breaked) {
                    if (breaked) {
                        next(new common_1.Exception("--break", "Middleware breaked the handling chain."));
                    }
                    else {
                        next(err);
                    }
                });
            }, function (err) {
                if (err) {
                    if (err.name === "--break") {
                        resp.finished || resp.end();
                        return;
                    }
                    else {
                        return this.emit(err.name, [req, resp, this], function () {
                            resp.finished || resp.end();
                        });
                    }
                }
                this.route(req, resp);
            }.bind(this));
        }
        else {
            this.route(req, resp);
        }
    };
    Server.prototype.route = function (req, resp) {
        if (req.handler) {
            var _this_1 = this;
            req.handler(req, resp, this, function (err) {
                if (err) {
                    _this_1.emit(err.name, [req, resp, this], function (err) {
                        resp.finished || resp.end();
                    });
                }
                else {
                    resp.finished || resp.end();
                }
            });
        }
        else {
            this.emit("/http/404", [req, resp, this], function (err) {
                resp.finished || resp.end();
            });
        }
    };
    return Server;
}(ServerProperties));
exports.Server = Server;
//# sourceMappingURL=Server.js.map