"use strict";
/**
 * A smart router-rule, supporting variable extraction.
 */
var SmartRouteRule = (function () {
    function SmartRouteRule(expr) {
        var replacement = [];
        var keys = [];
        this.oriExpr = expr;
        expr = RegExp.escape(expr.replace(/\{\s*\w+\s*:\s*\w+\s*\}/g, function (el) {
            var matchResult = el.match(/\{\s*(\w+)\s*:\s*(\w+)\s*\}/);
            switch (matchResult[2]) {
                case "int":
                    replacement.push("([-\\+]?\\d+)");
                    break;
                case "uint":
                    replacement.push("(\\d+)");
                    break;
                case "hex":
                    replacement.push("([\\dA-fa-f]+)");
                    break;
                case "string":
                    replacement.push("([^\\/]+)");
                    break;
                case "any":
                    replacement.push("(.+)");
                    break;
                default:
                    throw {
                        "name": "BAD-ROUTE-VARIABLE",
                        "message": "Invalid type of variable in routing rule."
                    };
            }
            keys.push(matchResult[1]);
            return "@::#" + (replacement.length - 1) + "#";
        }));
        for (var index = 0; index < replacement.length; index++) {
            expr = expr.replace("@::#" + index + "#", replacement[index]);
        }
        this.expr = new RegExp(expr);
        this.keys = keys;
    }
    Object.defineProperty(SmartRouteRule.prototype, "rawExpression", {
        get: function () {
            return this.oriExpr;
        },
        enumerable: true,
        configurable: true
    });
    SmartRouteRule.prototype.match = function (uri, data) {
        var ms = uri.match(this.expr);
        if (ms) {
            for (var x = 1; x < ms.length; x++) {
                data[this.keys[x - 1]] = ms[x];
            }
            return true;
        }
        return false;
    };
    return SmartRouteRule;
}());
module.exports = SmartRouteRule;
//# sourceMappingURL=SmartRouteRule.js.map