"use strict";
var NodePath = require("path");
var NodeFS = require("fs");
var common_1 = require("../../common");
var DEFAULT_BUFFER_SIZE = 8096;
exports.staticMineTypes = {
    ".png": {
        "MINE": "image/png"
    },
    ".gif": {
        "MINE": "image/gif"
    },
    ".jpg": {
        "MINE": "image/jpeg"
    },
    ".jpeg": {
        "MINE": "image/jpeg"
    },
    ".bmp": {
        "MINE": "image/bmp"
    },
    ".svg": {
        "MINE": "image/svg+xml"
    },
    ".ico": {
        "MINE": "image/x-icon"
    },
    ".tar": {
        "MINE": "application/x-tar"
    },
    ".tgz": {
        "MINE": "application/x-compressed"
    },
    ".txt": {
        "MINE": "text/plain"
    },
    ".json": {
        "MINE": "application/json",
        "cachable": true
    },
    ".zip": {
        "MINE": "application/zip"
    },
    ".html": {
        "MINE": "text/html",
        "cachable": true
    },
    ".htm": {
        "MINE": "text/html",
        "cachable": true
    },
    ".css": {
        "MINE": "text/css",
        "cachable": true
    },
    ".js": {
        "MINE": "application/x-javascript",
        "cachable": true
    },
    "default": {
        "MINE": "application/octet-stream"
    }
};
function StaticDelegate(options) {
    var memStore;
    if (options.prefix[options.prefix.length - 1] !== "/") {
        options.prefix += "/";
    }
    if (options.root[options.root.length - 1] !== "/") {
        options.root += "/";
    }
    var expires;
    if (options.expires && options.expires > 0) {
        expires = Math.ceil(1000 * options.expires);
    }
    return function (req, resp, next) {
        console.log("Detecting: " + req.url);
        if (!req.path.startsWith(options.prefix) || req.path.length === options.prefix.length) {
            return next();
        }
        if (NodePath.normalize(req.path)[0] === ".") {
            return next(new common_1.Exception("/http/404", "Suspected URL detected."));
        }
        console.log("Static Request: " + req.url);
        var memStore = options.memoryStorage.table;
        if (memStore) {
            var cn = memStore.get(req.path);
            if (cn) {
                console.log("Static Cache Hit.");
                resp.setHeader("Content-Type", cn.mine.MINE);
                resp.setHeader("Content-Length", cn.data.length.toString());
                if (expires) {
                    resp.setHeader("Expires", new Date(Date.now() + expires).toUTCString());
                    resp.setHeader("Cache-Control", "public, max-age=" + options.expires);
                }
                resp.end(cn.data);
                return next(null, true);
            }
        }
        var fn = options.root + req.path.substr(options.prefix.length);
        var mine;
        mine = exports.staticMineTypes[NodePath.extname(fn)] || exports.staticMineTypes["default"];
        NodePath.normalize(options.root + req.path.substr(options.prefix.length));
        console.log("Static Loading \"" + fn + "\".");
        NodeFS.readFile(fn, function (err, data) {
            if (err) {
                return next(new common_1.Exception("/http/404", "File not found."));
            }
            resp.setHeader("Content-Type", mine.MINE);
            resp.setHeader("Content-Length", data.length.toString());
            if (expires) {
                resp.setHeader("Expires", new Date(Date.now() + expires).toUTCString());
                resp.setHeader("Cache-Control", "public, max-age=" + options.expires);
            }
            resp.end(data);
            if (memStore && mine.cachable) {
                memStore.set(req.url, {
                    "mine": mine,
                    "data": data
                });
                console.log("\"" + req.path + "\"\" cached.");
            }
            next(null, true);
        });
    };
}
exports.StaticDelegate = StaticDelegate;
//# sourceMappingURL=StaticDelegate.js.map