"use strict";
var common_1 = require("../common");
var Logger = (function () {
    function Logger() {
        this.timeGenerator = function () {
            return new Date().toUTCString();
        };
        this.writers = {};
    }
    Object.defineProperty(Logger.prototype, "timeFormatter", {
        get: function () {
            return this.timeGenerator;
        },
        /**
         * Set the formatter of time.
         */
        set: function (newVal) {
            this.timeGenerator = newVal;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Register a new type of logging method.
     *
     * If a one existed, the old one will be overwritten.
     */
    Logger.prototype.registerType = function (type, writer) {
        this.writers[type] = writer;
        return this;
    };
    /**
     * Write a piece of log, and get the result in async way, if necessary.
     */
    Logger.prototype.write = function (data, type, next) {
        var writer = this.writers[type ? type : "default"];
        if (writer) {
            writer.call(this, data, function (err) {
                next && setImmediate(next, err);
            });
        }
        else {
            next && setImmediate(next, new common_1.Exception("NO-DEFAULT-WRITER", "The writer of default type doesn't exist."));
        }
        return this;
    };
    return Logger;
}());
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map