"use strict";
var NodeNet = require("net");
require("langext");
var client = new NodeNet.Socket();
/**
 *   Byte/     0       |       1       |       2       |       3       |
 *      /              |               |               |               |
 *     |0 1 2 3 4 5 6 7|0 1 2 3 4 5 6 7|0 1 2 3 4 5 6 7|0 1 2 3 4 5 6 7|
 *     +---------------+---------------+---------------+---------------+
 *    0| Magic         | Opcode        | Key length                    |
 *     +---------------+---------------+---------------+---------------+
 *    4| Extras length | Data type     | vbucket id                    |
 *     +---------------+---------------+---------------+---------------+
 *    8| Total body length                                             |
 *     +---------------+---------------+---------------+---------------+
 *   12| Opaque                                                        |
 *     +---------------+---------------+---------------+---------------+
 *   16| CAS                                                           |
 *     |                                                               |
 *     +---------------+---------------+---------------+---------------+
 *     Total 24 bytes
 */
var MagicCode;
(function (MagicCode) {
    MagicCode[MagicCode["REQUEST"] = 128] = "REQUEST";
    MagicCode[MagicCode["RESPONSE"] = 129] = "RESPONSE";
})(MagicCode || (MagicCode = {}));
var OpCodes;
(function (OpCodes) {
    OpCodes[OpCodes["GET"] = 0] = "GET";
    OpCodes[OpCodes["SET"] = 1] = "SET";
    OpCodes[OpCodes["ADD"] = 2] = "ADD";
    OpCodes[OpCodes["REPLACE"] = 3] = "REPLACE";
    OpCodes[OpCodes["DELETE"] = 4] = "DELETE";
    OpCodes[OpCodes["INCREMENT"] = 5] = "INCREMENT";
    OpCodes[OpCodes["DECREMENT"] = 6] = "DECREMENT";
})(OpCodes || (OpCodes = {}));
var MC_HEADER_LENGTH = 24;
function createRequestHeader(opcode, key, valueLength, vBucket, extraLength) {
    if (vBucket === void 0) { vBucket = 0; }
    if (extraLength === void 0) { extraLength = 0; }
    var ret = new Buffer(24);
    var keyBuf = new Buffer(key);
    ret.writeUInt8(MagicCode.REQUEST, 0);
    ret.writeUInt8(opcode, 1);
    ret.writeUInt16BE(keyBuf.byteLength, 2);
    ret.writeUInt8(extraLength, 4);
    ret.writeUInt8(0, 5);
    ret.writeUInt16BE(vBucket, 6);
    ret.writeUInt32BE(keyBuf.byteLength + extraLength + valueLength, 8);
    ret.writeUInt32BE(123321, 12);
    ret.writeUInt32BE(0, 16);
    ret.writeUInt32BE(0, 20);
    return ret;
}
var StatusCode;
(function (StatusCode) {
    StatusCode[StatusCode["NO_ERROR"] = 0] = "NO_ERROR";
    StatusCode[StatusCode["NOT_FOUND"] = 1] = "NOT_FOUND";
    StatusCode[StatusCode["DUP_KEY"] = 2] = "DUP_KEY";
    StatusCode[StatusCode["DATE_TOO_LONG"] = 3] = "DATE_TOO_LONG";
    StatusCode[StatusCode["BAD_ARGUMENTS"] = 4] = "BAD_ARGUMENTS";
    StatusCode[StatusCode["NOT_STORED"] = 5] = "NOT_STORED";
    StatusCode[StatusCode["NON_NUMERIC"] = 6] = "NON_NUMERIC";
    StatusCode[StatusCode["BAD_VBUCKET"] = 7] = "BAD_VBUCKET";
    StatusCode[StatusCode["AUTH_ERROR"] = 8] = "AUTH_ERROR";
    StatusCode[StatusCode["AUTH_CONTINUE"] = 9] = "AUTH_CONTINUE";
    StatusCode[StatusCode["UNKNOWN_CMD"] = 129] = "UNKNOWN_CMD";
    StatusCode[StatusCode["OUT_OF_MEMORY"] = 130] = "OUT_OF_MEMORY";
    StatusCode[StatusCode["NOT_SUPPORTED"] = 131] = "NOT_SUPPORTED";
    StatusCode[StatusCode["INTERNAL_ERROR"] = 132] = "INTERNAL_ERROR";
    StatusCode[StatusCode["BUSY"] = 133] = "BUSY";
    StatusCode[StatusCode["TEMP_FAILURE"] = 134] = "TEMP_FAILURE";
})(StatusCode || (StatusCode = {}));
/**
 *   Byte/     0       |       1       |       2       |       3       |
 *      /              |               |               |               |
 *     |0 1 2 3 4 5 6 7|0 1 2 3 4 5 6 7|0 1 2 3 4 5 6 7|0 1 2 3 4 5 6 7|
 *     +---------------+---------------+---------------+---------------+
 *    0| Magic         | Opcode        | Key Length                    |
 *     +---------------+---------------+---------------+---------------+
 *    4| Extras length | Data type     | Status                        |
 *     +---------------+---------------+---------------+---------------+
 *    8| Total body length                                             |
 *     +---------------+---------------+---------------+---------------+
 *   12| Opaque                                                        |
 *     +---------------+---------------+---------------+---------------+
 *   16| CAS                                                           |
 *     |                                                               |
 *     +---------------+---------------+---------------+---------------+
 *     Total 24 bytes
 */
var ResponsePack = (function () {
    function ResponsePack() {
    }
    return ResponsePack;
}());
function unpackResponsePack(pack) {
    var ret = new ResponsePack();
    ret.magic = pack.readUInt8(0);
    ret.opcode = pack.readUInt8(1);
    ret.keyLength = pack.readUInt16BE(2);
    ret.extraLength = pack.readUInt8(4);
    ret.dataType = pack.readUInt8(5);
    ret.status = pack.readUInt16BE(6);
    ret.opaque = pack.readUInt32BE(12);
    ret.cas = pack.readUIntBE(16, 8);
    ret.dataLength = pack.readUInt32BE(8) - ret.keyLength - ret.extraLength;
    if (ret.extraLength) {
        ret.extras = pack.slice(MC_HEADER_LENGTH, MC_HEADER_LENGTH + ret.extraLength);
    }
    if (ret.keyLength) {
        var keyData = pack.slice(MC_HEADER_LENGTH + ret.extraLength, MC_HEADER_LENGTH + ret.extraLength + ret.keyLength);
        ret.key = keyData.toString();
        keyData = undefined;
    }
    if (ret.dataLength) {
        ret.data = pack.slice(MC_HEADER_LENGTH + ret.extraLength + ret.keyLength, MC_HEADER_LENGTH + ret.extraLength + ret.keyLength + ret.dataLength);
    }
    return ret;
}
var PORT = 11211;
var HOST = "127.0.0.1";
client.connect(PORT, HOST, function () {
    console.log("CONNECTED TO: " + HOST + ":" + PORT);
    // 建立连接后立即向服务器发送数据，服务器将收到这些数据 
    //*
    client.write(createRequestHeader(OpCodes.GET, "abc", 0));
    client.write(new Buffer("abc"));
    return; //*/
    var extras = new Buffer(8);
    extras.writeUInt32BE(0x12345678, 0);
    extras.writeUInt32BE(30, 4);
    client.write(createRequestHeader(OpCodes.SET, "abc", 12, 0, extras.byteLength));
    client.write(extras);
    client.write(new Buffer("abc"));
    client.write(new Buffer("hello world!"));
    // client.write(new Buffer("abc"));
});
client.on("error", function (err) {
    console.log("ERROR", err);
});
// 为客户端添加“data”事件处理函数
// data是服务器发回的数据
client.on("data", function (data) {
    var unpackedData = unpackResponsePack(data);
    console.log(unpackedData);
    unpackedData.data && console.log(unpackedData.data.toString("utf-8"));
    // 完全关闭连接
    client.destroy();
});
// 为客户端添加“close”事件处理函数
client.on("close", function () {
    console.log("Connection closed");
});
//# sourceMappingURL=memcached.js.map