import { events, Exception } from "../lrts";
import async = require("async");

/**
 * A class based on the synchronous EventEmitter.
 */
class SyncEventTester extends events.SyncEventEmitter {

    public constructor() {

        super();
    }

    public test1(): void {

        console.log("sync test1");
    }

    public test2(): void {

        console.log("sync test2");
    }

    public test3(): void {

        console.log("sync test3");
    }
}

/**
 * A class based on the asynchronous EventEmitter.
 */
class AsyncEventTester extends events.AsyncEventEmitter {

    public constructor() {

        super();
    }

    public test1(): void {

        console.log("async test1");
    }

    public test2(): void {

        console.log("async test2");
    }

    public test3(): void {

        console.log("async test3");
    }
}

/**
 * Test the synchronous class.
 */
let SEE: SyncEventTester = new SyncEventTester();

SEE.listen("test", function(this: SyncEventTester, a: number): void {

    console.log(a);

    this.test1();
});

SEE.listenOnce("test", function(this: SyncEventTester, a: number): void {

    console.log(a);

    this.test2();
});

SEE.listenOnce("test", function(this: SyncEventTester, a: number): boolean {

    console.log("break", a);

    this.test3();

    return false; // break event transferring chain.
});

SEE.listen("test", function(this: SyncEventTester, a: number): void {

    console.log(a);

    this.test3();
});

/**
 * Test the asynchronous class.
 */
let AEE: AsyncEventTester = new AsyncEventTester();

AEE.on("hello", function(this: AsyncEventTester, a: number, next: events.AsyncEventChainCallback): void {

    console.log(a);

    this.test1();

    next(a === 3 ? new Exception("Hello-Exception", "This is a test for event error.") : null);

});

AEE.on("hello", function(this: AsyncEventTester, a: number, next: events.AsyncEventChainCallback): void {

    console.log(a);

    this.test2();

    next(null, a === 2); // break the transferring chain.

});

AEE.once("hello", function(this: AsyncEventTester, a: number, next: events.AsyncEventChainCallback): void {

    console.log("once", a);

    this.test3();

    next();

});

AEE.on("hello", function(this: AsyncEventTester, a: number, next: events.AsyncEventChainCallback): void {

    console.log("last", a);

    next();

});

/**
 * Begin Test
 */
AEE.emit("hello", [1], function(err?: Exception) {

    if (err) {

        console.log(err);
    }
});

AEE.emit("hello", [2], function(err?: Exception) {

    if (err) {

        console.log(err);
    }
});

AEE.emit("hello", [3], function(err?: Exception) {

    if (err) {

        console.log(err);
    }
});

SEE.emit("test", [1]);
SEE.emit("test", [2]);
SEE.emit("test", [3]);
