import "langext";

import NodeQS = require("querystring");

import { Exception, StdAsyncCallback, events, cache } from "../lrts";

import { Server, ServerRequest, ServerResponse, middlewares } from "../modules/http";

let server: Server = Server.create({
    "host": "127.0.0.1",
    "port": 80
});

server.get("/", function(request: ServerRequest, response: ServerResponse, server: Server, next: ErrorCallback) {
    response.setHeader("Content-Type", "text/html");
    response.write(`<!doctype html>
<html lang="zh_CN">
    <head>
        <meta charset="utf-8">
        <title>Test</title>
    </head>
    <body>
        <script src="static/modules/common.js"></script>
        <script src="static/modules/cache/index.js"></script>
        Hello world!
    </body>
</html>
`);
    next();
});

server.get("/user", function(request: ServerRequest, response: ServerResponse, server: Server, next: ErrorCallback) {

    response.write("who are you?");
    next();
});

server.get("/user/{id:uint}", function(request: ServerRequest, response: ServerResponse, server: Server, next: ErrorCallback) {

    if (request.query["action"]) {

        response.write(`hello, you uid is ${request.params["id"]}, do you wanna ${request.query["action"]}?`);

    } else {

        response.write(`hello, you uid is ${request.params["id"]}`);
    }
    next();
});

server.on("/http/404", function(request: ServerRequest, response: ServerResponse, server: Server, next: events.AsyncEventChainCallback) {

    response.statusCode = 404;
    response.statusMessage = "NOT FOUND";
    response.end("FILE NOT FOUND.");
    next();

});

cache.memory.create("httpStatic");

server.use(middlewares.QueryParser);
server.use(middlewares.StaticDelegate({
    "prefix": "/static/",
    "root": "./",
    "expires": 86400,
    "memoryStorage": {
        "table": cache.memory.get("httpStatic")
    }
}));

server.start(function(err?: Exception): void {
    if (err) {

        console.log("Error: Failed to listen to " + server.host + ":" + server.port + ".");
    } else {

        console.log("Server started with listening to " + server.host + ":" + server.port);
    }
});
